use std::fs;
use std::io::Read;
use std::process::Command;
use std::process::Stdio;

use super::Executor;

use super::helper::*;

#[derive(Debug, Clone, Copy)]
pub struct PythonExecutor {}

impl Executor for PythonExecutor {
    fn available(&self) -> bool {
        let cmd_ok = cmd_exists("python");
        if cmd_ok {
            let version_check =
                Command::new("python")
                    .arg("--version")
                    .stdout(Stdio::piped())
                    .spawn();
            if let Ok(version_check) = version_check {
                if let Some(mut sout) = version_check.stdout {
                    let mut buf = String::new();
                    let res = sout.read_to_string(&mut buf);
                    if res.is_err() {
                        return false;
                    }
                    
                    let is_py3 = buf.starts_with("Python 3");
                    if ! is_py3 {
                        log::warn!("`python` command is not a Python 3 interpreter.")
                    }
                    return is_py3
                }
            }
        }

        false
    }

    fn id(&self) -> String {
        "python".into()
    }

    fn exec(&self, code: String) -> Result<String, String> {
        assure_dir_exists().map_err(|x| x.to_string())?;

        let path = create_path();

        // 1. Save the code
        fs::write(path.clone(), code).map_err(|x| x.to_string())?;

        // 2. Run
        let mut out = Command::new("python")
            .arg(path.clone())
            .stdout(Stdio::piped())
            .spawn()
            .unwrap();

        // 3. Grab the stuff and exit
        let exit_status = out.wait().map_err(|x| x.to_string())?;

        let mut stdout: String = "".into();

        if let Some(mut sout) = out.stdout {
            let mut buf = vec![];
            sout.read_to_end(&mut buf).map_err(|x| x.to_string())?;
            stdout += String::from_utf8(buf).map_err(|x| x.to_string())?.as_str();
        } else {
            stdout += "Couldn't get stdout from the process"
        }

        stdout += "\n\n";
        stdout += exit_code_msg(exit_status).as_str();

        Ok(stdout)
    }
}